@@ -1,26 +1,35 @@
-from .common import TOOAPI_Baseclass,xrtmodes,convert_obsnum
+from .common import TOOAPI_Baseclass, TOOAPI_Daterange, TOOAPI_SkyCoord, TOOAPI_ObsID, xrtmodes
 from .too_status import Swift_TOO_Status
 from datetime import timedelta
-from tabulate import tabulate
-
-xrtmodes = {0: "Auto", 1: "Null", 2: "ShortIM", 3: "LongIM", 4: "PUPD", 5: "LRPD", 6: "WT", 7: "PC", 8: "Raw", 9: "Bias"}
-modesxrt = {"Auto": 0, "Null": 1, "ShortIM": 2, "LongIM": 3, "PUPD":4, "LRPD": 5 , "WT": 6, "PC": 7, "Raw": 8, "Bias": 9}
 
 
-class Swift_AFST_Entry(TOOAPI_Baseclass):
+class Swift_AFST_Entry(TOOAPI_Baseclass,TOOAPI_SkyCoord,TOOAPI_ObsID):
     '''Class that defines an individual entry in the Swift As-Flown Timeline'''
     def __init__(self):
         TOOAPI_Baseclass.__init__(self)
+        TOOAPI_SkyCoord.__init__(self)
+        TOOAPI_ObsID.__init__(self)
         self.api_name = "Swift_AFST_Entry"
-        # Entries
+        # Attributes of the class and their descriptions
         self.rows = ['begin','settle','end','ra','dec','roll','targname','targetid','seg','ra_point','dec_point','xrt','uvot','bat','fom','obstype']
+        self.names =['Begin Time','Settle Time','End Time','RA(J2000)','Dec(J200)','Roll (deg)','Target Name','Target ID','Segment','Object RA(J2000)','Object Dec(J2000)','XRT Mode','UVOT Mode','BAT Mode','Figure of Merit','Observation Type']
+        self.varnames = dict()
+        for i in range(len(self.rows)):
+            self.varnames[self.rows[i]] = self.names[i]
+        self.varnames['obsnum'] = 'Observation Number'
+        self.varnames['exposure'] = 'Exposure (s)'
+        self.varnames['slewtime'] = 'Slewtime (s)'
+        self.varnames['ra_object'] = self.varnames['ra_point']
+        self.varnames['dec_object'] = self.varnames['dec_point']
+
+        # Attributes
         self.begin = None
         self.settle = None
         self.end = None
         self.ra = None
         self.dec = None
-        self.ra_point = None
-        self.dec_point = None
+        self.ra_object = None
+        self.dec_object = None
         self.roll = None
         self.targname = None
         self.targetid = None
@@ -36,8 +45,8 @@ class Swift_AFST_Entry(TOOAPI_Baseclass):
     # Instrument modes
     @property
     def xrt(self):
-        """Given a XRT mode number returns a string containing the name of the
-        mode"""
+        '''Given a XRT mode number returns a string containing the name of the
+        mode'''
         return xrtmodes[self._xrt]
 
     @xrt.setter
@@ -46,30 +55,14 @@ class Swift_AFST_Entry(TOOAPI_Baseclass):
 
     @property
     def uvot(self):
-        """Given a XRT mode number returns a string containing the name of the
-        mode"""
+        '''Given a XRT mode number returns a string containing the name of the
+        mode'''
         return f"0x{self._uvot:04x}"
 
     @uvot.setter
     def uvot(self,mode):
         self._uvot = mode
 
-    # Handle the two different ways of reporting Observation number. Default to the SDC style of a string with leading zeros
-    @property
-    def obsnum(self):
-        '''Return the obsnum in SDC format'''
-        return f"{self.targetid:08d}{self.seg:03d}"
-    
-    @property 
-    def obsnumsc(self):
-        '''Return the obsnum in spacecraft format'''
-        return self.targetid + (self.seg<<24)
-    
-    @obsnum.setter
-    def obsnum(self,obsnum):
-        '''Set the obsnum value, by figuring out what the two formats are.'''
-        self._obsnum = convert_obsnum(obsnum)
-
     @property
     def exposure(self):
         return self.end - self.settle
@@ -77,34 +70,45 @@ class Swift_AFST_Entry(TOOAPI_Baseclass):
     @property
     def slewtime(self):
         return self.settle - self.begin
+
+    ## The following provides compatibility as we changed ra/dec_point to ra/dec_object. These will go away with a future API update.
+    @property
+    def ra_point(self):
+        return self.ra_object
     
+    @ra_point.setter
+    def ra_point(self,ra):
+        self.ra_object = ra
+
+    @property
+    def dec_point(self):
+        return self.dec_object
+
+    @dec_point.setter
+    def dec_point(self,dec):
+        self.dec_object = dec
+    ## Compat end 
+
     @property
     def table(self):
-        return [self.begin,self.end, self.targname, self.obsnum, self.exposure.seconds, self.slewtime.seconds]
-
-    def _repr_html_(self):
-        if self.table == []:
-            return "No data"
-        else:
-            return tabulate([self.table],['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='html',stralign='right').replace('right','left')
-    
-    def __str__(self):
-        if self.table == []:
-            return "No data"
-        else:
-            return tabulate([self.table],['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='pretty',stralign='right')
+        rows = ['begin','end','targname','obsnum','exposure','slewtime']
+        header = [self.varnames[row] for row in rows]
+        return header,[[self.begin,self.end, self.targname, self.obsnum, self.exposure.seconds, self.slewtime.seconds]]
 
 
-    
 
 class Swift_Observation(TOOAPI_Baseclass):
-    '''Class to package up and summarize all observations for a given observation ID (obsnum)'''
+    '''Class to summarize observations taken for given observation ID (obsnum).
+    Whereas observations are typically one or more individual snapshot, in TOO
+    API speak a `Swift_AFST_Entry`, this class summarizes all snapshots into a
+    single begin time, end time. Note that as ra/dec varies between each
+    snapshot, only `ra_object`, `dec_object` are given as coordinates. '''
     def __init__(self):
         TOOAPI_Baseclass.__init__(self)
         self.api_name = "Swift_Observation"
         # All the Swift_AFST_Entries for this observation
         self.entries = Swift_AFST()
-        self.rows = ['begin','end','targname','targetid','seg','ra_point','dec_point','xrt','uvot','entries']
+        self.rows = ['begin','end','targname','targetid','seg','ra_object','dec_object','xrt','uvot','entries']
         self.extrarows = []
 
     def __getitem__(self,index):
@@ -136,12 +140,14 @@ class Swift_Observation(TOOAPI_Baseclass):
         return self.entries[0].targname
 
     @property 
-    def ra_point(self):
-        return self.entries[0].ra_point
+    def ra_object(self):
+        if hasattr(self.entries[0], 'ra_object'):
+            return self.entries[0].ra_object
 
     @property 
-    def dec_point(self):
-        return self.entries[0].dec_point
+    def dec_object(self):
+        if hasattr(self.entries[0], 'dec_object'):
+            return self.entries[0].dec_object
 
     @property
     def exposure(self):
@@ -171,69 +177,73 @@ class Swift_Observation(TOOAPI_Baseclass):
     def snapshots(self):
         return self.entries
 
-    def __str__(self):
-        return f"{self.begin} - {self.end} Target: {self.targname:15s} ({self.obsnum}) Exp: {self.exposure.seconds:>5}s Slewtime: {self.slewtime.seconds:>5}s"
+    ## The following provides compatibility as we changed ra/dec_point to ra/dec_object. These will go away in the next version of the API (1.3).
+    @property
+    def ra_point(self):
+        return self.ra_object
+    
+    @ra_point.setter
+    def ra_point(self,ra):
+        self.ra_object = ra
+
+    @property
+    def dec_point(self):
+        return self.dec_object
+
+    @dec_point.setter
+    def dec_point(self,dec):
+        self.dec_object = dec
+    ## Compat end
 
     @property
     def table(self):
-        return [self.begin,self.end, self.targname, self.obsnum, self.exposure.seconds, self.slewtime.seconds]
-
-    def _repr_html_(self):
-        if self.table == []:
-            return "No data"
+        if len(self.entries) > 0:
+            header = self.entries[0].table[0]
         else:
-            return tabulate([self.table],['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='html',stralign='right').replace('right','left')
-    
-    def __str__(self):
-        if self.table == []:
-            return "No data"
-        else:
-            return tabulate([self.table],['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='pretty',stralign='right')
+            header = []
+        return header,[[self.begin, self.end, self.targname, self.obsnum, self.exposure.seconds, self.slewtime.seconds]]
 
-class Swift_Observations(dict):
-    '''Adapted dictionary class for containing observations that mostly is just to ensure that data can be displayed in a consistent format.
-    Key is typically the Swift Observation ID in SDC format (e.g. '00012345012').'''
+
+class Swift_Observations(dict,TOOAPI_Baseclass):
+    '''Adapted dictionary class for containing observations that mostly is just
+    to ensure that data can be displayed in a consistent format. Key is
+    typically the Swift Observation ID in SDC format (e.g. '00012345012').'''
 
     @property
     def table(self):
-        return [self[obsid].table for obsid in self.keys()]
-
-    def _repr_html_(self):
-        if self.table == []:
-            return "No data"
+        if len(self.values()) > 0:
+            header = list(self.values())[0].table[0]
         else:
-            return tabulate(self.table,['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='html',stralign='right').replace('right','left')
-    
-    def __str__(self):
-        if self.table == []:
-            return "No data"
-        else:
-            return tabulate(self.table,['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='pretty',stralign='right')
+            header = []
+        return header,[self[obsid].table[1][0] for obsid in self.keys()]
 
 
-class Swift_AFST(TOOAPI_Baseclass):
-    '''Class to fetch Swift As-Flown Science Timeline (AFST) for given constraints. Essentially this will
-    return what Swift observed and when, for given constraints. Constraints can be for give coordinate
-    (SkyCoord or J2000 RA/Dec) and radius (in degrees), a given date range, or a given target ID (targetid) 
-    or Observation ID (obsnum).'''
-    def __init__(self,username=None,shared_secret=None,ra=None,dec=None,begin=None,end=None,targetid=None,obsnum=None,radius=0.1967):
+class Swift_AFST(TOOAPI_Baseclass,TOOAPI_Daterange,TOOAPI_SkyCoord,TOOAPI_ObsID):
+    '''Class to fetch Swift As-Flown Science Timeline (AFST) for given
+    constraints. Essentially this will return what Swift observed and when, for
+    given constraints. Constraints can be for give coordinate (SkyCoord or J2000
+    RA/Dec) and radius (in degrees), a given date range, or a given target ID
+    (targetid) or Observation ID (obsnum).'''
+    def __init__(self,username='anonymous',shared_secret=None,ra=None,dec=None,begin=None,end=None,targetid=None,obsnum=None,radius=0.1967,length=None):
         TOOAPI_Baseclass.__init__(self)
+        TOOAPI_Daterange.__init__(self)
+        TOOAPI_ObsID.__init__(self)
         self.api_name = "Swift_AFST"
         # Coordinate search
-        self._skycoord = None
         self.ra = ra
         self.dec = dec
         self.radius = radius # Default 11.8 arcmin - XRT FOV
         # begin and end boundaries        
         self.begin = begin
-        self.end = end
+        self.end = end    
+        self.length = length
         # Search on targetid/obsnum
         self.targetid = targetid
-        self._obsnum = None
         self.obsnum = obsnum        
         # Login
         self.username = username
-        self.shared_secret = shared_secret
+        if shared_secret != None:
+            self.shared_secret = shared_secret
         # AFST entries go here
         self.entries = list()
         # Status of request
@@ -241,44 +251,23 @@ class Swift_AFST(TOOAPI_Baseclass):
         # AFST maximum date
         self.afstmax = None
         # Contents of the rows
-        self.rows = ['username','begin','end','ra','dec','radius','targetid','obsnum','afstmax','entries']
-        self.extrarows = ['status']
+        self.rows = ['username','begin','end','ra','dec','radius','targetid','obsnum']
+        self.extrarows = ['status','afstmax','entries']
         self.trans_name = dict()
         # Acceptable classes that be part of this class
         self.subclasses = [Swift_AFST_Entry,Swift_TOO_Status]
         # Observations
         self._observations = Swift_Observations()
-        if self.username != None:
+        if self.ra != None or self.begin != None or targetid != None or obsnum != None:
             self.submit()
 
-    @property
-    def obsnum(self):
-        return self._obsnum
-
-    @obsnum.setter
-    def obsnum(self,obsnum):
-        '''Allow obsnum to be specified in Spacecraft (int) or SDC format (string), or an array of either'''
-        if type(obsnum) == list or type(obsnum) == tuple:
-            self._obsnum = [convert_obsnum(obs) for obs in obsnum]
-        else: 
-            self._obsnum = convert_obsnum(obsnum)
-
     @property
     def table(self):
-        return [ppt.table for ppt in self]
-
-    def _repr_html_(self):
-        if self.table == []:
-            return "No data"
+        if len(self.entries) > 0:
+            header = self.entries[0].table[0]
         else:
-            return tabulate(self.table,['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='html',stralign='right').replace('right','left')
-    
-    def __str__(self):
-        if self.table == []:
-            return "No data"
-        else:
-            return tabulate(self.table,['Begin','End','Name','Obs Number','Exposure (s)','Slewtime (s)'],tablefmt='pretty',stralign='right')
-
+            header = []
+        return header,[ppt.table[1][0] for ppt in self]
 
     @property
     def observations(self):
@@ -288,23 +277,6 @@ class Swift_AFST(TOOAPI_Baseclass):
             _ = [self._observations[q.obsnum].append(q) for q in self.entries]
         return self._observations
 
-    @property 
-    def skycoord(self): 
-        # Check if the RA/Dec match the SkyCoord, and if they don't modify the skycoord
-        if type(self._skycoord).__module__ == 'astropy.coordinates.sky_coordinate':
-            if self.ra != self._skycoord.fk5.ra.deg or self.dec != self._skycoord.fk5.dec.deg:
-                self._skycoord = self._skycoord.__class__(self.ra,self.dec,unit="deg",frame="fk5")
-        return self._skycoord
-
-    @skycoord.setter
-    def skycoord(self,sc):
-        if type(sc).__module__ == 'astropy.coordinates.sky_coordinate':
-            self._skycoord = sc
-            self.ra = sc.fk5.ra.deg
-            self.dec = sc.fk5.dec.deg
-        else:
-            raise Exception("Needs to be assigned an astropy SkyCoord")
-
     def __getitem__(self,index):
         return self.entries[index]
 
